@@ -200,6 +200,7 @@ class bookmarks {
         }
 
         // destination is an array
+        // Named destinations handled by PdfReader::consolidateNamedDestinations
         if (destination_p != null && destination_p.isArray()) {
 
           ArrayList<PdfObject> array_list_p = ((PdfArray) destination_p).getArrayList();
@@ -208,14 +209,17 @@ class bookmarks {
             PdfObject page_p = reader_p.getPdfObject(array_list_p.get(0));
             if (page_p != null && page_p.isDictionary()) {
               bookmark.m_page_num = GetPageNumber((PdfDictionary) page_p, reader_p, cache) + 1;
+            } else if (page_p != null && page_p.isNumber()) {
+              // page_p should always be a PdfDictionary (cf PDF Reference 1.7 Section 12.3.2.2)
+              // but seen PdfNumber in the wild (cf pdftk-java issue 123)
+              bookmark.m_page_num = ((PdfNumber) page_p).intValue() + 1;
             } else { // error
               fail_b = true;
             }
           } else { // error
             fail_b = true;
           }
-        } // TODO: named destinations handling
-        else { // error
+        } else { // error
           fail_b = true;
         }
 
