@@ -58,18 +58,27 @@ class cat {
     m_output_keep_final_id_b = session.m_output_keep_final_id_b;
   }
 
-  static char GetPdfVersionChar(PdfName version_p) {
+  static char GetPdfVersionChar(PdfObject version_p) {
     char version_cc = PdfWriter.VERSION_1_4; // default
 
-    if (version_p != null)
-      if (version_p.equals(PdfName.VERSION_1_4)) version_cc = PdfWriter.VERSION_1_4;
-      else if (version_p.equals(PdfName.VERSION_1_5)) version_cc = PdfWriter.VERSION_1_5;
-      else if (version_p.equals(PdfName.VERSION_1_6)) version_cc = PdfWriter.VERSION_1_6;
-      else if (version_p.equals(PdfName.VERSION_1_7)) version_cc = PdfWriter.VERSION_1_7;
-      else if (version_p.equals(PdfName.VERSION_1_3)) version_cc = PdfWriter.VERSION_1_3;
-      else if (version_p.equals(PdfName.VERSION_1_2)) version_cc = PdfWriter.VERSION_1_2;
-      else if (version_p.equals(PdfName.VERSION_1_1)) version_cc = PdfWriter.VERSION_1_1;
-      else if (version_p.equals(PdfName.VERSION_1_0)) version_cc = PdfWriter.VERSION_1_0;
+    if (version_p == null) return version_cc;
+
+    if (!version_p.isName()) {
+      try {
+        version_p = new PdfName(version_p.toString());
+      } catch (Exception e) {
+        return version_cc;
+      }
+    }
+
+    if (version_p.equals(PdfName.VERSION_1_4)) version_cc = PdfWriter.VERSION_1_4;
+    else if (version_p.equals(PdfName.VERSION_1_5)) version_cc = PdfWriter.VERSION_1_5;
+    else if (version_p.equals(PdfName.VERSION_1_6)) version_cc = PdfWriter.VERSION_1_6;
+    else if (version_p.equals(PdfName.VERSION_1_7)) version_cc = PdfWriter.VERSION_1_7;
+    else if (version_p.equals(PdfName.VERSION_1_3)) version_cc = PdfWriter.VERSION_1_3;
+    else if (version_p.equals(PdfName.VERSION_1_2)) version_cc = PdfWriter.VERSION_1_2;
+    else if (version_p.equals(PdfName.VERSION_1_1)) version_cc = PdfWriter.VERSION_1_1;
+    else if (version_p.equals(PdfName.VERSION_1_0)) version_cc = PdfWriter.VERSION_1_0;
 
     return version_cc;
   }
@@ -194,7 +203,7 @@ class cat {
       PdfDictionary catalog_p = reader_p.getCatalog();
       if (catalog_p.contains(PdfName.VERSION)) {
 
-        PdfName version_p = (PdfName) reader_p.getPdfObject(catalog_p.get(PdfName.VERSION));
+        PdfObject version_p = reader_p.getPdfObject(catalog_p.get(PdfName.VERSION));
         char version_cc = GetPdfVersionChar(version_p);
 
         if (max_version_cc < version_cc) max_version_cc = version_cc;
@@ -211,7 +220,19 @@ class cat {
           // iterate over developers
           Set<PdfObject> keys_p = extensions_p.getKeys();
           for (PdfObject kit : keys_p) {
-            PdfName developer_p = (PdfName) reader_p.getPdfObject(kit);
+            // We need to be defensive about object types
+            // cf https://gitlab.com/pdftk-java/pdftk/-/issues/127
+            PdfObject developer_po = reader_p.getPdfObject(kit);
+            PdfName developer_p = null;
+            if (developer_po.isName()) {
+              developer_p = (PdfName) developer_po;
+            } else {
+              try {
+                developer_p = new PdfName(developer_po.toString());
+              } catch (Exception e) {
+                continue;
+              }
+            }
 
             PdfObject dev_exts_po = reader_p.getPdfObject(extensions_p.get(developer_p));
             if (dev_exts_po != null && dev_exts_po.isDictionary()) {
@@ -221,22 +242,43 @@ class cat {
                   && dev_exts_p.contains(PdfName.EXTENSIONLEVEL)) {
                 // use the greater base version or the greater extension level
 
-                PdfName base_version_p =
-                    (PdfName) reader_p.getPdfObject(dev_exts_p.get(PdfName.BASEVERSION));
-                PdfNumber ext_level_p =
-                    (PdfNumber) reader_p.getPdfObject(dev_exts_p.get(PdfName.EXTENSIONLEVEL));
+                PdfObject base_version_po =
+                    reader_p.getPdfObject(dev_exts_p.get(PdfName.BASEVERSION));
+                PdfName base_version_p = null;
+                if (base_version_po.isName()) {
+                  base_version_p = (PdfName) base_version_p;
+                } else {
+                  try {
+                    base_version_p = new PdfName(base_version_po.toString());
+                  } catch (Exception e) {
+                    continue;
+                  }
+                }
+
+                PdfObject ext_level_po =
+                    reader_p.getPdfObject(dev_exts_p.get(PdfName.EXTENSIONLEVEL));
+                Integer ext_level_i = null;
+                if (ext_level_po.isNumber()) {
+                  ext_level_i = ((PdfNumber) ext_level_po).intValue();
+                } else {
+                  try {
+                    ext_level_i = Integer.valueOf(ext_level_i.toString());
+                  } catch (Exception e) {
+                    continue;
+                  }
+                }
 
                 if (!ext_base_versions.containsKey(developer_p)
                     || GetPdfVersionChar(ext_base_versions.get(developer_p))
-                        < GetPdfVersionChar(base_version_p)) {
+                        < GetPdfVersionChar(base_version_po)) {
                   // new developer or greater base version
                   ext_base_versions.put(developer_p, base_version_p);
-                  ext_levels.put(developer_p, ext_level_p.intValue());
+                  ext_levels.put(developer_p, ext_level_i);
                 } else if (GetPdfVersionChar(ext_base_versions.get(developer_p))
-                        == GetPdfVersionChar(base_version_p)
-                    && ext_levels.get(developer_p) < ext_level_p.intValue()) {
+                        == GetPdfVersionChar(base_version_po)
+                    && ext_levels.get(developer_p) < ext_level_i) {
                   // greater extension level for current base version
-                  ext_levels.put(developer_p, ext_level_p.intValue());
+                  ext_levels.put(developer_p, ext_level_i);
                 }
               }
             }
