@@ -735,6 +735,10 @@ public class PdfStamperImp extends PdfWriter {
                 if (ff != null)
                     flags = ff.intValue();
                 int page = ((Integer)item.page.get(k)).intValue();
+                // Page might be out of bounds
+                // See https://gitlab.com/pdftk-java/pdftk/-/issues/134
+                if (page < 1 || page > reader.getNumberOfPages())
+                    continue;
                 PdfDictionary appDic = (PdfDictionary)PdfReader.getPdfObject(merged.get(PdfName.AP));
                 if (appDic != null && (flags & PdfFormField.FLAGS_PRINT) != 0 && (flags & PdfFormField.FLAGS_HIDDEN) == 0) {
                     PdfObject obj = appDic.get(PdfName.N);
@@ -1120,7 +1124,7 @@ public class PdfStamperImp extends PdfWriter {
         }
     }
     
-    void addAnnotation(PdfAnnotation annot, int page) {
+    public void addAnnotation(PdfAnnotation annot, int page) {
         addAnnotation(annot, reader.getPageN(page));
     }
 
