@@ -27,16 +27,23 @@ import com.gitlab.pdftk_java.com.lowagie.text.FontFactory;
 import com.gitlab.pdftk_java.com.lowagie.text.Rectangle;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.AcroFields;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.FdfReader;
+import com.gitlab.pdftk_java.com.lowagie.text.pdf.PRIndirectReference;
+import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfAnnotation;
+import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfArray;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfBoolean;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfContentByte;
+import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfCopy;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfDictionary;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfImportedPage;
+import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfIndirectReference;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfName;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfNumber;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfObject;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfReader;
+import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfRectangle;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.PdfStamperImp;
 import com.gitlab.pdftk_java.com.lowagie.text.pdf.XfdfReader;
+import java.awt.geom.AffineTransform;
 import java.io.ByteArrayInputStream;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
@@ -138,7 +145,13 @@ class filter {
           // maybe it's xfdf?
           try {
             xfdf_reader_p = new XfdfReader(new ByteArrayInputStream(buffer));
-          } catch (IOException ioe2_p) { // file open error
+          } catch (IOException | RuntimeException ioe2_p) { // file open error
+            // A RuntimeException may occur if the input looks like an
+            // XML file but does not contain an XFDF tag. Probably it
+            // should be an IOException for consistency for
+            // SimpleXMLDocHandler, and certainly not an unchecked
+            // exception. See
+            // https://gitlab.com/pdftk-java/pdftk/-/issues/125
             System.err.println("Error: Failed read form data on stdin.");
             return ErrorCode.ERROR;
           }
@@ -347,6 +360,9 @@ class filter {
       PdfImportedPage mark_page_p = null;
       Rectangle mark_page_size_p = null;
       int mark_page_rotation = 0;
+      ArrayList<PdfDictionary> mark_annots = new ArrayList();
+      PdfCopy copy = new PdfCopy(writer_p);
+      copy.setFromReader(mark_p);
 
       // iterate over document's pages, adding mark_page as
       // a layer above (stamp) or below (watermark) the page content;
@@ -367,6 +383,9 @@ class filter {
           // create a PdfTemplate from the first page of mark
           // (PdfImportedPage is derived from PdfTemplate)
           mark_page_p = writer_p.getImportedPage(mark_p, ii);
+
+          PdfDictionary mark_full_page_p = mark_p.getPageN(ii);
+          mark_annots = report.getAnnots(mark_p, mark_full_page_p);
         }
 
         // the target page geometry
@@ -394,35 +413,65 @@ class filter {
         PdfContentByte content_byte_p =
             (background_b) ? writer_p.getUnderContent(ii) : writer_p.getOverContent(ii);
 
+        float[] trans = null;
         if (mark_page_rotation == 0) {
-          content_byte_p.addTemplate(mark_page_p, mark_scale, 0, 0, mark_scale, h_trans, v_trans);
+          trans = new float[] {mark_scale, 0, 0, mark_scale, h_trans, v_trans};
         } else if (mark_page_rotation == 90) {
-          content_byte_p.addTemplate(
-              mark_page_p,
-              0,
-              -1 * mark_scale,
-              mark_scale,
-              0,
-              h_trans,
-              v_trans + mark_page_size_p.height() * mark_scale);
+          trans =
+              new float[] {
+                0,
+                -1 * mark_scale,
+                mark_scale,
+                0,
+                h_trans,
+                v_trans + mark_page_size_p.height() * mark_scale
+              };
         } else if (mark_page_rotation == 180) {
-          content_byte_p.addTemplate(
-              mark_page_p,
-              -1 * mark_scale,
-              0,
-              0,
-              -1 * mark_scale,
-              h_trans + mark_page_size_p.width() * mark_scale,
-              v_trans + mark_page_size_p.height() * mark_scale);
+          trans =
+              new float[] {
+                -1 * mark_scale,
+                0,
+                0,
+                -1 * mark_scale,
+                h_trans + mark_page_size_p.width() * mark_scale,
+                v_trans + mark_page_size_p.height() * mark_scale
+              };
         } else if (mark_page_rotation == 270) {
+          trans =
+              new float[] {
+                0,
+                mark_scale,
+                -1 * mark_scale,
+                0,
+                h_trans + mark_page_size_p.width() * mark_scale,
+                v_trans
+              };
+        }
+        if (trans != null) {
           content_byte_p.addTemplate(
-              mark_page_p,
-              0,
-              mark_scale,
-              -1 * mark_scale,
-              0,
-              h_trans + mark_page_size_p.width() * mark_scale,
-              v_trans);
+              mark_page_p, trans[0], trans[1], trans[2], trans[3], trans[4], trans[5]);
+        }
+
+        for (PdfDictionary annot : mark_annots) {
+          PdfAnnotation new_annot = new PdfAnnotation(writer_p, null);
+          new_annot.putAll(annot);
+          for (Object key : new_annot.getKeys()) {
+            Object value = new_annot.get((PdfName) key);
+            if (value instanceof PRIndirectReference) {
+              PdfIndirectReference ind = copy.copyIndirect((PRIndirectReference) value);
+              new_annot.put((PdfName) key, ind);
+            }
+          }
+          PdfArray rect = (PdfArray) annot.get(PdfName.RECT);
+          float[] rect_f = new float[4];
+          for (int i = 0; i < 4; ++i) rect_f[i] = rect.getAsNumber(i).floatValue();
+          AffineTransform M = new AffineTransform(trans);
+          float[] new_rect_f = new float[4];
+          M.transform(rect_f, 0, new_rect_f, 0, 2);
+          PdfRectangle new_rect =
+              new PdfRectangle(new_rect_f[0], new_rect_f[1], new_rect_f[2], new_rect_f[3], 0);
+          new_annot.put(PdfName.RECT, new_rect);
+          writer_p.addAnnotation(new_annot, ii);
         }
       }
     }
