@@ -21,8 +21,8 @@ public class DataFieldsTest extends BlackBox {
 
   @Test
   public void dump_data_fields_utf8_options() throws IOException {
-    pdftk("test/files/form-utf8.pdf", "dump_data_fields_utf8");
-    String expectedData = slurp("test/files/form-utf8.data");
+    pdftk("test/files/form-pdfdocencoding.pdf", "dump_data_fields_utf8");
+    String expectedData = slurp("test/files/form-pdfdocencoding.data");
     assertEquals(expectedData, systemOut.getLog());
   }
 };
