@@ -22,6 +22,13 @@ public class DataTest extends BlackBox {
     assertEquals(expectedData, systemOut.getLog());
   }
 
+  @Test
+  public void dump_data_invalid_outlines() throws IOException {
+    pdftk("test/files/issue123.pdf", "dump_data_utf8");
+    String expectedData = slurp("test/files/issue123.data");
+    assertEquals(expectedData, systemOut.getLog());
+  }
+
   @Test
   public void idempotent() {
     pdftk("test/files/refs.pdf", "dump_data_utf8");
